@@ -3,7 +3,7 @@
 #
 # Natural Language Toolkit: TGrep search
 #
-# Copyright (C) 2001-2016 NLTK Project
+# Copyright (C) 2001-2018 NLTK Project
 # Author: Will Roberts <wildwilhelm@gmail.com>
 # URL: <http://nltk.org/>
 # For license information, see LICENSE.TXT
@@ -13,7 +13,9 @@ Unit tests for nltk.tgrep.
 '''
 
 from __future__ import absolute_import, print_function, unicode_literals
-from nltk.compat import b
+
+from six import b
+
 from nltk.tree import ParentedTree
 from nltk import tgrep
 import unittest
